@@ -2,8 +2,8 @@
 # Licensed under the terms of the Apache License, Version 2.0. See the LICENSE file associated with the project for terms.
 """Lightweight :term:`computation` graphs for Python."""
 
-__version__ = "6.2.0"
-__release_date__ = "19 Apr 2020, 14:03"
+__version__ = "7.0.0"
+__release_date__ = "28 Apr 2020, 18:57"
 __title__ = "graphtik"
 __summary__ = __doc__.splitlines()[0]
 __license__ = "Apache-2.0"
@@ -14,7 +14,7 @@ __author__ = "hnguyen, ankostis"
 from .base import NO_RESULT
 from .config import (
     abort_run,
-    debug,
+    debug_enabled,
     evictions_skipped,
     execution_pool_plugged,
     get_execution_pool,
