@@ -1,12 +1,15 @@
 # Copyright 2016, Yahoo Inc.
 # Licensed under the terms of the Apache License, Version 2.0. See the LICENSE file associated with the project for terms.
 """
-Generic or specific utilities  without polluting imports.
+Generic utilities, or :mod:`.plot` API but without polluting imports.
 
-.. testsetup::
+.. doctest::
+    :hide:
 
-    from graphtik.base import *
-    __name__ = "graphtik.base"
+    .. Workaround sphinx-doc/sphinx#6590
+
+    >>> from graphtik.base import *
+    >>> __name__ = "graphtik.base"
 """
 
 import abc
@@ -373,11 +376,32 @@ class PlotArgs(NamedTuple):
     #: the list of output names .
     outputs: Collection = None
     #: Contains the computed results, which might be different from :attr:`plottable`.
-    solution: "Solution" = None
+    solution: "graphtik.network.Solution" = None
     #: a mapping of nodes to cluster names
     clusters: Mapping = None
-    #: if given, overrides :active plotter`
-    plotter: "Plotter" = None
+    #: If given, overrides :active plotter`.
+    plotter: "graphtik.plot.Plotter" = None
+    #: If given, overrides :term:`plot theme` plotter will use.
+    theme: "graphtik.plot.Theme" = None
+
+    #######
+    # Internal item-args for :meth:`.Plotter._make_node()` etall.
+    #
+    #: Where to add graphviz nodes & stuff.
+    dot: "pydot.Dot" = None
+    #: The node (data(str) or :class:`Operation`) or edge as gotten from nx-graph.
+    nx_item: Any = None
+    #: Attributes gotten from nx-graph for the given graph/node/edge.
+    #: They are NOT a clone, so any modifications affect the nx `graph`.
+    nx_attrs: dict = None
+    #: The pydot-node/edge created
+    dot_item: Any = None
+    #: Collect the actual clustered `dot_nodes` among the given nodes.
+    clustered: dict = None
+
+    #######
+    # Render args
+    #
     #: jupyter configuration overrides
     jupyter_render: Mapping = None
     #: where to write image or show in a matplotlib window
@@ -427,14 +451,15 @@ class Plottable(abc.ABC):
         filename: Union[str, bool, int] = None,
         show=None,
         *,
-        plotter: "Plotter" = None,
+        plotter: "graphtik.plot.Plotter" = None,
+        theme: "graphtik.plot.Theme" = None,
         graph: "networkx.Graph" = None,
         name=None,
         steps=None,
         inputs=None,
         outputs=None,
-        solution=None,
-        clusters=None,
+        solution: "graphtik.network.Solution" = None,
+        clusters: Mapping = None,
         jupyter_render: Union[None, Mapping, str] = None,
     ) -> "pydot.Dot":
         """
@@ -460,7 +485,7 @@ class Plottable(abc.ABC):
             the :term:`plottable` that ordered the plotting.
             Automatically set downstreams to one of::
 
-                netop | net | plan | solution | <missing>
+                op | netop | net | plan | solution | <missing>
 
             :seealso: :attr:`.PlotArgs.plottable`
         :param plotter:
@@ -468,6 +493,11 @@ class Plottable(abc.ABC):
             is used by default.
 
             :seealso: :attr:`.PlotArgs.plotter`
+        :param theme:
+            Any :term:`plot theme` overrides; if none, the :attr:`.Plotter.default_theme`
+            of the :term:`active plotter` is used.
+
+            :seealso: :attr:`.PlotArgs.theme`
         :param name:
             if not given, dot-lang graph would is named "G"; necessary to be unique
             when referring to generated CMAPs.
@@ -493,7 +523,7 @@ class Plottable(abc.ABC):
 
             - "public" attributes: reaching `Graphviz`_ as-is, e.g.
               to set ``spline -> ortho`` on the graph attributes
-              (this can also be achieved by modified :term:`plot styles`).
+              (this can also be achieved by modified :term:`plot theme`).
 
               .. Note::
 
@@ -606,9 +636,11 @@ class Plottable(abc.ABC):
         digraph netop {
         fontname=italic;
         label=<netop>;
-        <a> [fillcolor=wheat, shape=invhouse, style=filled, tooltip="(int) 1"];
+        <a> [fillcolor=wheat, margin="0.02,0.02", shape=invhouse, style=filled, tooltip="(int) 1"];
         ...
 
+        .. graphtik::
+
         .. TODO: move advanced plot examples from API --> tutorial page
 
         You may use the :attr:`PlotArgs.graph` overlay to skip certain nodes (or edges)
