@@ -8,8 +8,15 @@
 
     Plot configrations were not defined here, not to pollute import space early,
     until they are actually needed.
+
+.. note::
+    The contant-manager function ``XXX_plugged()`` or ``XXX_enabled()`` do NOT launch
+    their code blocks using :meth:`contextvars.Context.run()` in a separate "context",
+    so any changes to these or other context-vars will persist
+    (unless they are also done within such context-managers)
 """
 import ctypes
+import os
 from contextlib import contextmanager
 from contextvars import ContextVar
 from functools import partial
@@ -18,7 +25,11 @@ from typing import Optional
 
 from boltons.iterutils import first
 
-_debug: ContextVar[Optional[bool]] = ContextVar("debug", default=False)
+_debug_env_var = os.environ.get("GRAPHTIK_DEBUG")
+_debug: ContextVar[Optional[bool]] = ContextVar(
+    "debug",
+    default=_debug_env_var and (_debug_env_var.lower() not in "0 false off no".split()),
+)
 _abort: ContextVar[Optional[bool]] = ContextVar(
     "abort", default=Value(ctypes.c_bool, lock=False)
 )
@@ -54,15 +65,31 @@ def _tristate_armed(context_var: ContextVar, enabled):
         context_var.reset(resetter)
 
 
-debug = partial(_tristate_armed, _debug)
+debug_enabled = partial(_tristate_armed, _debug)
 """
 Like :func:`set_debug()` as a context-manager, resetting back to old value.
+
+.. seealso:: disclaimer about context-managers the top of this :mod:`.config` module.
 """
 is_debug = partial(_getter, _debug)
 """see :func:`set_debug()`"""
 set_debug = partial(_tristate_set, _debug)
 """
-When true, string-representation of network objects and errors become more detailed.
+When true, increase details on string-representation of network objects and errors.
+
+:param enabled:
+    - ``None, False, string(0, false, off, no)``: Disabled
+    - anything else: Enable ALL ``DEBUG_XXX``
+
+Affected behavior:
+
++ net objects print details recursively;
++ plotted SVG diagrams include style-provenance as tooltips;
++ Sphinx extension also saves the original DOT file next to each image
+  (see :confval:`graphtik_save_dot_files`).
+
+.. Note::
+    The default is controlled with :envvar:`GRAPHTIK_DEBUG` environment variable.
 
 Note that enabling this flag is different from enabling logging in DEBUG,
 since it affects all code (eg interactive printing in debugger session,
@@ -96,6 +123,8 @@ def is_abort():
 evictions_skipped = partial(_tristate_armed, _skip_evictions)
 """
 Like :func:`set_skip_evictions()` as a context-manager, resetting back to old value.
+
+.. seealso:: disclaimer about context-managers the top of this :mod:`.config` module.
 """
 is_skip_evictions = partial(_getter, _skip_evictions)
 """see :func:`set_skip_evictions()`"""
@@ -114,6 +143,8 @@ regardless of asked outputs.
 def execution_pool_plugged(pool: "Optional[Pool]"):
     """
     Like :func:`set_execution_pool()` as a context-manager, resetting back to old value.
+
+    .. seealso:: disclaimer about context-managers the top of this :mod:`.config` module.
     """
     resetter = _execution_pool.set(pool)
     try:
@@ -140,6 +171,8 @@ def get_execution_pool() -> "Optional[Pool]":
 tasks_in_parallel = partial(_tristate_armed, _parallel_tasks)
 """
 Like :func:`set_parallel_tasks()` as a context-manager, resetting back to old value.
+
+.. seealso:: disclaimer about context-managers the top of this :mod:`.config` module.
 """
 is_parallel_tasks = partial(_getter, _parallel_tasks)
 """see :func:`set_parallel_tasks()`"""
@@ -159,6 +192,8 @@ Enable/disable globally :term:`parallel` execution of operations.
 tasks_marshalled = partial(_tristate_armed, _marshal_tasks)
 """
 Like :func:`set_marshal_tasks()` as a context-manager, resetting back to old value.
+
+.. seealso:: disclaimer about context-managers the top of this :mod:`.config` module.
 """
 is_marshal_tasks = partial(_getter, _marshal_tasks)
 """see :func:`set_marshal_tasks()`"""
@@ -180,6 +215,8 @@ inputs & outputs with :mod:`dill`,  which might help for pickling problems.
 operations_endured = partial(_tristate_armed, _endure_operations)
 """
 Like :func:`set_endure_operations()` as a context-manager, resetting back to old value.
+
+.. seealso:: disclaimer about context-managers the top of this :mod:`.config` module.
 """
 is_endure_operations = partial(_getter, _endure_operations)
 """see :func:`set_endure_operations()`"""
@@ -200,6 +237,8 @@ Enable/disable globally :term:`endurance` to keep executing even if some operati
 operations_reschedullled = partial(_tristate_armed, _reschedule_operations)
 """
 Like :func:`set_reschedule_operations()` as a context-manager, resetting back to old value.
+
+.. seealso:: disclaimer about context-managers the top of this :mod:`.config` module.
 """
 is_reschedule_operations = partial(_getter, _reschedule_operations)
 """see :func:`set_reschedule_operations()`"""
