@@ -27,7 +27,7 @@ from .config import (
     is_skip_evictions,
     is_solid_true,
 )
-from .modifiers import optional, sideffect, vararg, varargs
+from .modifiers import mapped, optional, sideffect, vararg, varargs
 from .op import Operation
 
 NodePredicate = Callable[[Any, Mapping], bool]
@@ -117,7 +117,7 @@ def _unsatisfied_operations(dag, inputs: Collection) -> List:
                 else:
                     # Prune operations with partial inputs.
                     unsatisfied.append(node)
-        elif isinstance(node, (_DataNode, str)):  # `str` are givens
+        elif isinstance(node, str):  # `str` are givens
             if node in ok_data:
                 # mark satisfied-needs on all future operations
                 for future_op in dag.adj[node]:
@@ -152,8 +152,14 @@ class Solution(ChainMap, Plottable):
         (after the :meth:`finalized` has been invoked)
     """
 
-    def __init__(self, plan, input_values):
-        super().__init__(input_values)
+    def __init__(self, plan, input_values: dict):
+        #: An ordered mapping of plan-operations to their results
+        #: (initially empty dicts).
+        #: The result dictionaries pre-populate this (self) chainmap,
+        #: with the 1st map (wins all reads) the last operation,
+        #: the last one the `input_values` dict.
+        self._layers = {op: {} for op in yield_ops(reversed(plan.steps))}
+        super().__init__(*self._layers.values(), input_values)
 
         self.plan = plan
         self.executed = {}
@@ -162,12 +168,6 @@ class Solution(ChainMap, Plottable):
         self.elapsed_ms = {}
         self.solid = "%X" % random.randint(0, 2 ** 16)
 
-        ## Pre-populate chainmaps with 1 dict per plan's operation
-        #  (appended after of inputs map).
-        #
-        self._layers = {op: {} for op in yield_ops(plan.steps)}
-        self.maps.extend(self._layers.values())
-
         ## Cache context-var flags.
         #
         self.is_endurance = is_endure_operations()
@@ -278,7 +278,6 @@ class Solution(ChainMap, Plottable):
         """invoked only once, after all ops have been executed"""
         # Invert solution so that last value wins
         if not self.finalized:
-            self.maps = self.maps[::-1]
             self.finalized = True
 
     def __delitem__(self, key):
@@ -321,7 +320,7 @@ class Solution(ChainMap, Plottable):
                 if pouts and isinstance(pouts, list)
             }
             err_msgs = [
-                f"\n  +--{op.name}: {type(ex).__name__}({ex})"
+                f"\n  +--{op.name}: {type(ex).__name__}('{ex}')"
                 for op, ex in failures.items()
             ]
             msg = (
@@ -347,17 +346,6 @@ class Solution(ChainMap, Plottable):
         return plot_args
 
 
-class _DataNode(str):
-    """
-    Dag node naming a data-value produced or required by an operation.
-    """
-
-    __slots__ = ()  # avoid __dict__ on instances
-
-    def __repr__(self):
-        return f"DataNode('{self}')"
-
-
 class _EvictInstruction(str):
     """
     A step in the ExecutionPlan to evict a computed value from the `solution`.
@@ -375,7 +363,7 @@ class _EvictInstruction(str):
 
 def _yield_datanodes(nodes):
     """May scan dag nodes."""
-    return (n for n in nodes if isinstance(n, _DataNode))
+    return (n for n in nodes if isinstance(n, str))
 
 
 def yield_ops(nodes):
@@ -383,17 +371,19 @@ def yield_ops(nodes):
     return (n for n in nodes if isinstance(n, Operation))
 
 
+def yield_node_names(nodes):
+    """Yield either ``op.name`` or ``str(node)``."""
+    return (str(getattr(n, "name", n)) for n in nodes)
+
+
 def _optionalized(graph, data):
     """Retain optionality of a `data` node based on all `needs` edges."""
     all_optionals = all(e[2] for e in graph.out_edges(data, "optional", False))
-    sideffector = graph.nodes(data="sideffect")
     return (
         optional(data)
         if all_optionals
-        # Nodes are _DataNode instances, not `optional` or `sideffect`
-        # TODO: Unify _DataNode + modifiers to avoid ugly hack `net.collect_requirements()`.
-        else sideffect(data)
-        if sideffector[data]
+        else data  # sideffect
+        if isinstance(data, sideffect)
         else str(data)  # un-optionalize
     )
 
@@ -401,10 +391,11 @@ def _optionalized(graph, data):
 def collect_requirements(graph) -> Tuple[iset, iset]:
     """Collect & split datanodes in (possibly overlapping) `needs`/`provides`."""
     operations = list(yield_ops(graph))
-    provides = iset(p for op in operations for p in op.provides)
+    provides = iset(
+        p for op in operations for p in getattr(op, "op_provides", op.provides)
+    )
     needs = iset(_optionalized(graph, n) for op in operations for n in op.needs)
-    # TODO: Unify _DataNode + modifiers to avoid ugly hack `net.collect_requirements()`.
-    provides = iset(str(n) if not isinstance(n, sideffect) else n for n in provides)
+    provides = iset(provides)
     return needs, provides
 
 
@@ -527,7 +518,7 @@ class ExecutionPlan(
         steps = (
             "".join(f"\n  +--{s}" for s in self.steps)
             if is_debug()
-            else ", ".join(str(getattr(s, "name", s)) for s in self.steps)
+            else ", ".join(yield_node_names(self.steps))
         )
         return f"ExecutionPlan(needs={needs}, provides={provides}, x{len(self.steps)} steps: {steps})"
 
@@ -989,26 +980,39 @@ class Network(Plottable):
         #
         needs = []
         needs_edges = []
-        for n in operation.needs:
+        for n in getattr(operation, "op_needs", operation.needs):
             nkw, ekw = {}, {}
             if isinstance(n, (optional, vararg, varargs)):
                 ekw["optional"] = True
             if isinstance(n, sideffect):
                 ekw["sideffect"] = nkw["sideffect"] = True
-            needs.append((_DataNode(n), nkw))
-            needs_edges.append((_DataNode(n), operation, ekw))
+            if isinstance(n, mapped) and n.fn_arg is not None:
+                ekw["_fn_arg"] = n.fn_arg
+            needs.append((n, nkw))
+            needs_edges.append((n, operation, ekw))
         graph.add_nodes_from(needs)
         graph.add_node(operation, **operation.node_props)
         graph.add_edges_from(needs_edges)
 
+        ## Prepare inversed-aliases index, used
+        #  to label edges reaching to aliased `provides`.
+        #
+        aliases = getattr(operation, "aliases", None)
+        alias_sources = {v: src for src, v in aliases} if aliases else ()
+
         ## Provides
         #
-        for n in operation.provides:
+        for n in getattr(operation, "op_provides", operation.provides):
             kw = {}
             if isinstance(n, sideffect):
                 kw["sideffect"] = True
-                graph.add_node(_DataNode(n), sideffect=True)
-            graph.add_edge(operation, _DataNode(n), **kw)
+                graph.add_node(n, sideffect=True)
+
+            if n in alias_sources:
+                src_provide = alias_sources[n]
+                kw["_alias_of"] = src_provide
+
+            graph.add_edge(operation, n, **kw)
 
     def _topo_sort_nodes(self, dag) -> List:
         """Topo-sort dag respecting operation-insertion order to break ties."""
@@ -1119,7 +1123,7 @@ class Network(Plottable):
             # unrelated nodes further up the dag.
             ending_in_outputs = set()
             for output_name in outputs:
-                ending_in_outputs.add(_DataNode(output_name))
+                ending_in_outputs.add(output_name)
                 ending_in_outputs.update(nx.ancestors(dag, output_name))
             broken_dag = broken_dag.subgraph(ending_in_outputs)
             if log.isEnabledFor(logging.INFO) and len(broken_dag) != len(dag):
@@ -1241,7 +1245,7 @@ class Network(Plottable):
 
             else:
                 assert isinstance(
-                    node, _DataNode
+                    node, str
                 ), f"Unrecognized network graph node {node}: {type(node).__name__!r}"
 
         return steps
