@@ -8,7 +8,8 @@ import logging
 import textwrap
 from collections import abc as cabc
 from collections import namedtuple
-from typing import Any, Callable, List, Mapping, Tuple, Union
+from functools import wraps
+from typing import Any, Callable, Collection, List, Mapping, Set, Tuple, Union
 
 from boltons.setutils import IndexedSet as iset
 
@@ -17,6 +18,7 @@ from .base import (
     UNSET,
     Items,
     MultiValueError,
+    PlotArgs,
     Plottable,
     aslist,
     astuple,
@@ -24,17 +26,25 @@ from .base import (
     jetsam,
 )
 from .config import is_debug, is_reschedule_operations, is_solid_true
-from .modifiers import arg, optional, sideffect, vararg, varargs
+from .modifiers import mapped, optional, sideffect, sol_sideffect, vararg, varargs
 
 log = logging.getLogger(__name__)
 
 
-def _dict_without(kw, *todel):
-    return {k: v for k, v in kw.items() if k not in todel}
+def _dict_without(kw, *to_skip):
+    return {k: v for k, v in kw.items() if k not in to_skip}
 
 
 def as_renames(i, argname):
-    """parses a list of (source-->destination) from dict, list-of-2-items, single 2-tuple."""
+    """
+    Parses a list of (source-->destination) from dict, list-of-2-items, single 2-tuple.
+
+    :return:
+        a (possibly empty)list-of-pairs
+
+    .. Note::
+        The same `source` may be repeatedly renamed to multiple `destinations`.
+    """
     if not i:
         return ()
 
@@ -59,12 +69,17 @@ def as_renames(i, argname):
     return i
 
 
-def reparse_operation_data(name, needs, provides):
+def reparse_operation_data(
+    name, needs, provides
+) -> Tuple[cabc.Hashable, cabc.Collection, cabc.Collection]:
     """
     Validate & reparse operation data as lists.
 
-    As a separate function to be reused by client code
-    when building operations and detect errors early.
+    :return:
+        name, needs, provides,
+
+    As a separate function to be reused by client building operations,
+    to detect errors early.
     """
 
     if not isinstance(name, cabc.Hashable):
@@ -86,6 +101,10 @@ def reparse_operation_data(name, needs, provides):
 class Operation(abc.ABC):
     """An abstract class representing an action with :meth:`.compute()`."""
 
+    @property
+    def __name__(self) -> str:
+        return self.name  # pylint: disable=no-member
+
     @abc.abstractmethod
     def compute(self, named_inputs, outputs=None):
         """
@@ -103,20 +122,74 @@ class Operation(abc.ABC):
         """
 
 
-class FunctionalOperation(Operation):
+def _spread_sideffects(
+    deps: cabc.Collection,
+) -> Tuple[cabc.Collection, cabc.Collection]:
+    """
+    Build fn/op dependencies from user ones by stripping or singularizing any :term:`sideffects`.
+
+    :return:
+        the given `deps` duplicated as ``(fn_deps,  op_deps)``, where any instances of
+        :class:`.sideffects` and :class:`.sol_sideffect` are processed like this:
+
+        `fn_deps`
+            - :class:`.sol_sideffect` are replaced by the pure :attr:`.sol_sideffect.sideffected`
+              consumed/produced by underlying functions, in the order it is first met
+              (the rest duplicate `sideffected` are discarded).
+            - :class:`.sideffects` are simply dropped;
+
+        `op_deps`
+            :class:`.sol_sideffect` are replaced by a sequence of "singular" `sol_sideffect`
+            instances, one for each item in their :attr:`.sol_sideffect.sideffects` attribute,
+            in the order they are first met
+            (any duplicates are discarded, order is irrelevant, since they don't reach
+            the function);
+    """
+
+    def singularize_sol_sideffects(dep):
+        return (
+            (sol_sideffect(dep.sideffected, s) for s in dep.sideffects)
+            if isinstance(dep, sol_sideffect)
+            else (dep,)
+        )
+
+    #: The only dupes that are dropped from `fn_deps` are any `sideffected`,
+    #: to facilitate copy-pasting singularized ones from the console.
+    seen_sideffecteds: Set[str] = set()
+
+    def strip_sideffecteds(dep):
+        if isinstance(dep, sol_sideffect):
+            sideffected = dep.sideffected
+            if not sideffected in seen_sideffecteds:
+                seen_sideffecteds.add(sideffected)
+                return (sideffected,)
+        elif not isinstance(dep, sideffect):
+            return (dep,)
+        return ()
+
+    assert deps is not None
+
+    if deps:
+        deps = tuple(nn for n in deps for nn in singularize_sol_sideffects(n))
+        fn_deps = tuple(nn for n in deps for nn in strip_sideffecteds(n))
+        return deps, fn_deps
+    else:
+        return deps, deps
+
+
+class FunctionalOperation(Operation, Plottable):
     """
     An :term:`operation` performing a callable (ie a function, a method, a lambda).
 
     .. Tip::
-        Use :class:`operation()` builder class to build instances of this class instead.
+        - Use :func:`.operation()` factory to build instances of this class instead.
+        - Call :meth:`withset()` on existing instances to re-configure new clones.
     """
 
     def __init__(
         self,
-        # def __new__(
-        # cls,
-        fn: Callable,
-        name,
+        fn: Callable = None,
+        name=None,
         needs: Items = None,
         provides: Items = None,
         aliases: Mapping = None,
@@ -132,68 +205,13 @@ class FunctionalOperation(Operation):
         """
         Build a new operation out of some function and its requirements.
 
-        See :class:`.operation` for the full documentation of parameters.
-
-        .. attribute:: FunctionalOperation.name
-
-            a name for the operation (e.g. `'conv1'`, `'sum'`, etc..);
-            it will be prefixed by `parents`.
-        .. attribute:: FunctionalOperation.needs
-
-            Names of input data objects this operation requires.
-        .. attribute:: FunctionalOperation.provides
-
-            Names of the **real output values** the underlying function provides
-            (without `aliases`, with(!) `sideffects`)
-
-            NOTE that the instance attribute eventually includes aliases & sideffects.
-        .. attribute:: FunctionalOperation.real_provides
-
-            Value names the underlying function provides (without aliases, with(!) sideffects).
-
-            FIXME: `real_provides` not sure what it does with sideffects.
-        .. attribute:: FunctionalOperation.aliases
-
-            an optional mapping of real `provides` to additional ones, together
-            comprising this operations :term:`provides`.
-        .. attribute:: FunctionalOperation.parents
-
-            a tuple wth the names of the parents, prefixing `name`,
-            but also kept for equality/hash check.
-        .. attribute:: FunctionalOperation.rescheduled
-
-            If true, underlying *callable* may produce a subset of `provides`,
-            and the :term:`plan` must then :term:`reschedule` after the operation
-            has executed.  In that case, it makes more sense for the *callable*
-            to `returns_dict`.
-        .. attribute:: FunctionalOperation.endured
-
-            If true, even if *callable* fails, solution will :term:`reschedule`;
-            ignored if :term:`endurance` enabled globally.
-        .. attribute:: FunctionalOperation.parallel
-
-            execute in :term:`parallel`
-        .. attribute:: FunctionalOperation.marshalled
-
-            If true, operation will be :term:`marshalled <marshalling>` while computed,
-            along with its `inputs` & `outputs`.
-            (usefull when run in `parallel` with a :term:`process pool`).
-        .. attribute:: FunctionalOperation.returns_dict
-
-            if true, it means the `fn` returns a dictionary with all `provides`,
-            and no further processing is done on them
-            (i.e. the returned output-values are not zipped with `provides`)
-        .. attribute:: FunctionalOperation.node_props
-
-            Added as-is into NetworkX graph, and you may filter operations by
-            :meth:`.NetworkOperation.withset()`.
-            Also plot-rendering affected if they match `Graphviz` properties,
-            unless they start with underscore(``_``).
+        See :func:`.operation` for the full documentation of parameters,
+        study the code for attributes (or read them from  rendered sphinx site).
         """
         super().__init__()
         node_props = node_props = node_props if node_props else {}
 
-        if not fn or not callable(fn):
+        if fn and not callable(fn):
             raise ValueError(f"Operation was not provided with a callable: {fn}")
         if parents and not isinstance(parents, tuple):
             raise ValueError(
@@ -204,42 +222,107 @@ class FunctionalOperation(Operation):
                 f"Operation `node_props` must be a dict, was {type(node_props).__name__!r}: {node_props}"
             )
 
-        ## Overwrite reparsed op-data.
-        if name is None:
+        if name is None and fn:
             name = func_name(fn, None, mod=0, fqdn=0, human=0)
-        name = ".".join(str(pop) for pop in ((parents or ()) + (name,)))
-        name, needs, real_provides = reparse_operation_data(name, needs, provides)
+        if name is not None:
+            name = ".".join(str(pop) for pop in ((parents or ()) + (name,)))
+        ## Overwrite reparsed op-data.
+        name, needs, provides = reparse_operation_data(name, needs, provides)
+
+        needs, _fn_needs = _spread_sideffects(needs)
+        provides, _fn_provides = _spread_sideffects(provides)
+        op_needs = iset(needs)
 
         if aliases:
             aliases = as_renames(aliases, "aliases")
-            alias_src, alias_dst = list(zip(*aliases))
-            full_provides = iset(itt.chain(real_provides, alias_dst))
-            if not set(alias_src) <= set(real_provides):
-                raise ValueError(
-                    f"Operation `aliases` contain sources not found in real `provides`: {list(iset(alias_src) - real_provides)}"
+            if any(1 for src, dst in aliases if dst in provides):
+                bad = ", ".join(
+                    f"{src} -> {dst}" for src, dst in aliases if dst in provides
                 )
-            if any(isinstance(i, sideffect) for i in alias_src) or any(
-                isinstance(i, sideffect) for i in alias_dst
-            ):
                 raise ValueError(
-                    f"Operation `aliases` must not contain `sideffects`: {aliases}"
+                    f"The `aliases` ({bad}) clash with existing provides {list(provides)}!"
+                )
+
+            alias_src, alias_dst = list(zip(*aliases))
+            if not set(alias_src) <= set(provides):
+                raise ValueError(
+                    f"The `aliases` for {list(alias_src)} rename {list(iset(alias_src) - provides)}"
+                    f", not found in provides {list(provides)}!"
+                )
+            sfx_aliases = [
+                f"{src} -> {dst}"
+                for src, dst in aliases
+                if isinstance(src, sideffect) or isinstance(dst, sideffect)
+            ]
+            if sfx_aliases:
+                raise ValueError(
+                    f"The `aliases` must not contain `sideffects` {sfx_aliases}"
                     "\n  Simply add any extra `sideffects` in the `provides`."
                 )
         else:
-            full_provides = real_provides
+            alias_dst = ()
+        op_provides = iset(itt.chain(provides, alias_dst))
 
         self.fn = fn
+        #: a name for the operation (e.g. `'conv1'`, `'sum'`, etc..);
+        #: it will be prefixed by `parents`.
         self.name = name
+
+        #: The :term:`needs` almost as given by the user
+        #: (which may contain MULTI-sol_sideffects and dupes),
+        #: roughly morphed into `_fn_provides` + sideffects
+        #: (dupes preserved, with sideffects & SINGULARIZED :term:`solution sideffect`\s).
+        #: It is stored for builder functionality to work.
         self.needs = needs
-        self.provides = full_provides
-        self.real_provides = real_provides
+        #: Value names ready to lay the graph for :term:`pruning`
+        #: (NO dupes, WITH aliases & sideffects, and SINGULAR :term:`solution sideffect`\s).
+        self.op_needs = op_needs
+        #: Value names the underlying function requires
+        #: (dupes preserved, without sideffects, with stripped :term:`sideffected` dependencies).
+        self._fn_needs = _fn_needs
+
+        #: The :term:`provides` almost as given by the user
+        #: (which may contain MULTI-sol_sideffects and dupes),
+        #: roughly morphed into `_fn_provides` + sideffects
+        #: (dupes preserved, without aliases, with sideffects & SINGULARIZED :term:`solution sideffect`\s).
+        #: It is stored for builder functionality to work.
+        self.provides = provides
+        #: Value names ready to lay the graph for :term:`pruning`
+        #: (NO dupes, WITH aliases & sideffects, and SINGULAR sol_sideffects).
+        self.op_provides = op_provides
+        #: Value names the underlying function produces
+        #: (dupes preserved, without aliases & sideffects, with stripped :term:`sideffected` dependencies).
+        self._fn_provides = _fn_provides
+        #: an optional mapping of `fn_provides` to additional ones, together
+        #: comprising this operations :term:`op_provides`.
+        #:
+        #: You cannot alias an :term:`alias`.
         self.aliases = aliases
+        #: a tuple wth the names of the parents, prefixing `name`,
+        #: but also kept for equality/hash check.
         self.parents = parents
+        #: If true, underlying *callable* may produce a subset of `provides`,
+        #: and the :term:`plan` must then :term:`reschedule` after the operation
+        #: has executed.  In that case, it makes more sense for the *callable*
+        #: to `returns_dict`.
         self.rescheduled = rescheduled
+        #: If true, even if *callable* fails, solution will :term:`reschedule`;
+        #: ignored if :term:`endurance` enabled globally.
         self.endured = endured
+        #: execute in :term:`parallel`
         self.parallel = parallel
+        #: If true, operation will be :term:`marshalled <marshalling>` while computed,
+        #: along with its `inputs` & `outputs`.
+        #: (usefull when run in `parallel` with a :term:`process pool`).
         self.marshalled = marshalled
+        #: if true, it means the `fn` returns a dictionary with all `provides`,
+        #: and no further processing is done on them
+        #: (i.e. the returned output-values are not zipped with `provides`)
         self.returns_dict = returns_dict
+        #: Added as-is into NetworkX graph, and you may filter operations by
+        #: :meth:`.NetworkOperation.withset()`.
+        #: Also plot-rendering affected if they match `Graphviz` properties,
+        #: unless they start with underscore(``_``).
         self.node_props = node_props
 
     def __eq__(self, other):
@@ -268,32 +351,58 @@ class FunctionalOperation(Operation):
         endured = "!" if self.endured else ""
         parallel = "|" if self.parallel else ""
         marshalled = "$" if self.marshalled else ""
+
+        if is_debug():
+            debug_needs = (
+                f", op_needs={list(self.op_needs)}, fn_needs={list(self._fn_needs)}"
+            )
+            debug_provides = f", op_provides={list(self.op_provides)}, fn_provides={list(self._fn_provides)}"
+        else:
+            debug_needs = debug_provides = ""
         return (
-            f"FunctionalOperation{endured}{parallel}{marshalled}(name={self.name!r}, needs={needs!r}, "
-            f"provides={provides!r}{resched}{aliases}, fn{returns_dict_marker}={fn_name!r}{nprops})"
+            f"FunctionalOperation{endured}{resched}{parallel}{marshalled}(name={self.name!r}, "
+            f"needs={needs!r}{debug_needs}, provides={provides!r}{debug_provides}{aliases}, "
+            f"fn{returns_dict_marker}={fn_name!r}{nprops})"
         )
 
-    def withset(self, **kw) -> "FunctionalOperation":
+    @property
+    def deps(self) -> Mapping[str, Collection]:
         """
-        Make a clone with the some values replaced.
+        All :term:`dependency` names, including `op_` & internal `_fn_`.
 
-        .. ATTENTION::
-            Using :meth:`.namedtuple._replace()` would not pass through cstor,
-            so would not get a nested `name` with `parents`, not arguments validation.
+        if not DEBUG, all deps are converted into lists, ready to be printed.
         """
-        # fn = kw["fn"] if "fn" in kw else self.fn
-        # name = kw["name"] if "name" in kw else self.name
-        # needs = kw["needs"] if "needs" in kw else self.needs
-        # provides = kw["provides"] if "provides" in kw else self.provides
-        # aliases = kw["aliases"] if "aliases" in kw else self.aliases
-        kw2 = vars(self).copy()
-        kw2["provides"] = kw2["real_provides"]
-        del kw2["real_provides"]
-        kw2.update(kw)
 
-        return FunctionalOperation(**kw2)
+        return {
+            k: v if is_debug() else list(v)
+            for k, v in zip(
+                "needs op_needs fn_needs provides op_provides fn_provides".split(),
+                (
+                    self.needs,
+                    self.op_needs,
+                    self._fn_needs,
+                    self.provides,
+                    self.op_provides,
+                    self._fn_provides,
+                ),
+            )
+        }
 
-    def _prepare_plot_args_error(
+    def withset(self, fn: Callable = None, **kw,) -> "FunctionalOperation":
+        """Make a *clone* with the some values replaced. """
+        ## Exclude calculated dep-fields.
+        #
+        me = {
+            k: v
+            for k, v in vars(self).items()
+            if not k.startswith("_") and not k.startswith("op_")
+        }
+        if fn:
+            me["fn"] = fn
+        me.update(kw)
+        return FunctionalOperation(**me)
+
+    def _prepare_match_inputs_error(
         self,
         exceptions: List[Tuple[Any, Exception]],
         missing: List,
@@ -388,7 +497,9 @@ class FunctionalOperation(Operation):
                 results = [results]
                 ngot = 1
 
-            else:  # nexpected > 1; nexpected == 0 was the very 1st check.
+            else:
+                # nexpected == 0 was method's 1st check.
+                assert nexpected > 1, nexpected
                 if isinstance(results, (str, bytes)) or not isinstance(
                     results, cabc.Iterable
                 ):
@@ -437,10 +548,17 @@ class FunctionalOperation(Operation):
 
     def compute(self, named_inputs, outputs=None) -> dict:
         try:
+            if self.fn is None:
+                raise ValueError(
+                    f"Operation was not yet provided with a callable `fn`!"
+                )
+            assert self.name is not None, self
+
             positional, vararg_vals = [], []
             kwargs = {}
             errors, missing, varargs_bad = [], [], []
-            for n in self.needs:
+            for n in self._fn_needs:
+                assert not isinstance(n, sideffect), locals()
                 try:
                     if n not in named_inputs:
                         if not isinstance(n, (optional, vararg, varargs, sideffect)):
@@ -454,7 +572,7 @@ class FunctionalOperation(Operation):
                     ## TODO: augment modifiers with "retrievers" from `inputs`.
                     inp_value = named_inputs[n]
 
-                    if isinstance(n, arg):  # includes `optionals`
+                    if isinstance(n, mapped):  # includes `optionals`
                         kwargs[n if n.fn_arg is None else n.fn_arg] = inp_value
 
                     elif isinstance(n, vararg):
@@ -468,11 +586,8 @@ class FunctionalOperation(Operation):
                         else:
                             vararg_vals.extend(i for i in inp_value)
 
-                    elif isinstance(n, sideffect):
-                        pass  # ignored as function argument.
-
                     else:
-                        positional.append(named_inputs[n])
+                        positional.append(inp_value)
 
                 except Exception as nex:
                     log.debug(
@@ -485,16 +600,14 @@ class FunctionalOperation(Operation):
                     errors.append((n, nex))
 
             if errors or missing or varargs_bad:
-                raise self._prepare_plot_args_error(
+                raise self._prepare_match_inputs_error(
                     errors, missing, varargs_bad, named_inputs
                 )
 
             results_fn = self.fn(*positional, *vararg_vals, **kwargs)
 
             # TODO: rename op jetsam (real_)provides --> fn_expected
-            provides = iset(
-                n for n in self.real_provides if not isinstance(n, sideffect)
-            )
+            provides = self._fn_provides
             results_op = self._zip_results_with_provides(results_fn, provides)
 
             if outputs:
@@ -524,33 +637,101 @@ class FunctionalOperation(Operation):
             raise
 
     def __call__(self, *args, **kwargs):
+        """
+        Although may return results like :meth:`compute()`, does no checks, and
+
+        it it passes args/kw as user desires.
+        """
         return self.fn(*args, **kwargs)
 
+    def prepare_plot_args(self, plot_args: PlotArgs) -> PlotArgs:
+        """Delegate to a provisional network with a single op . """
+        from .netop import compose
+        from .plot import graphviz_html_string
 
-class operation:
-    """
-    A builder for graph-operations wrapping functions.
+        is_user_label = bool(plot_args.graph and plot_args.graph.get("label"))
+        plottable = compose(self.name, self)
+        plot_args = plot_args.with_defaults(name=self.name)
+        plot_args = plottable.prepare_plot_args(plot_args)
+        assert plot_args.graph, plot_args
+
+        ## Operations don't need another name visible.
+        #
+        if not is_user_label:
+            del plot_args.graph.graph["label"]
+        plot_args = plot_args._replace(plottable=self)
+
+        return plot_args
+
+
+def operation(
+    fn: Callable = None,
+    name=None,
+    needs: Items = None,
+    provides: Items = None,
+    aliases: Mapping = None,
+    *,
+    rescheduled=None,
+    endured=None,
+    parallel=None,
+    marshalled=None,
+    returns_dict=None,
+    node_props: Mapping = None,
+):
+    r"""
+    An :term:`operation` factory that can function as a decorator.
 
     :param fn:
-        The callable underlying this operation.  This does not need to be
-        specified when the operation object is instantiated and can instead
-        be set via ``__call__`` later.
+        The callable underlying this operation.
+        If given, it builds the operation right away (along with any other arguments).
+
+        If not given, it returns a "fancy decorator" that still supports all arguments
+        here AND the ``withset()`` method.
+
+        .. hint::
+            This is a twisted way for `"fancy decorators"
+            <https://realpython.com/primer-on-python-decorators/#both-please-but-never-mind-the-bread>`_.
+
+        After all that, you can always call :meth:`FunctionalOperation.withset()`
+        on existing operation, to obtain a re-configured clone.
     :param str name:
         The name of the operation in the computation graph.
+        If not given, deduce from any `fn` given.
+
     :param needs:
-        The list of (positionally ordered) names of the data needed by the `operation`
+        the list of (positionally ordered) names of the data needed by the `operation`
         to receive as :term:`inputs`, roughly corresponding to the arguments of
-        the underlying `fn`.
+        the underlying `fn` (plus any :term:`sideffects`).
+
+        It can be a single string, in which case a 1-element iterable is assumed.
+
+        .. seealso::
+            - :term:`needs`
+            - :term:`modifier`
+            - :attr:`.FunctionalOperation.needs`
+            - :attr:`.FunctionalOperation.op_needs`
+            - :attr:`.FunctionalOperation._fn_needs`
+
 
-        See also :term:`needs` & :term:`modifiers`.
     :param provides:
-        Names of output data this operation provides, which must correspond
-        to the returned values of the `fn`.
-        If more than one given, those must be returned in an iterable,
-        unless `returns_dict` is true, in which case a dictionary with (at least)
-        as many elements must be returned.
+        the list of (positionally ordered) output data this operation provides,
+        which must, roughly, correspond to the returned values of the `fn`
+        (plus any :term:`sideffects` & :term:`alias`\es).
+
+        It can be a single string, in which case a 1-element iterable is assumed.
+
+        If they are more than one, the underlying function must return an iterable
+        with same number of elements, unless param `returns_dict` :term:`is true
+        <returns dictionary>`, in which case must return a dictionary that containing
+        (at least) those named elements.
+
+        .. seealso::
+            - :term:`provides`
+            - :term:`modifier`
+            - :attr:`.FunctionalOperation.provides`
+            - :attr:`.FunctionalOperation.op_provides`
+            - :attr:`.FunctionalOperation._fn_provides`
 
-        See also :term:`provides` & :term:`modifiers`.
     :param aliases:
         an optional mapping of `provides` to additional ones
     :param rescheduled:
@@ -577,154 +758,63 @@ class operation:
         unless they start with underscore(``_``)
 
     :return:
-        when called, it returns a :class:`.FunctionalOperation`
+        when called with `fn`, it returns a :class:`.FunctionalOperation`,
+        otherwise it returns a decorator function that accepts `fn` as the 1st argument.
+
+        .. Note::
+            Actually the returned decorator is the :meth:`.FunctionalOperation.withset()`
+            method and accepts all arguments, monkeypatched to support calling a virtual
+            ``withset()`` method on it, not to interrupt the builder-pattern,
+            but only that - besides that trick, it is just a bound method.
 
     **Example:**
 
     This is an example of its use, based on the "builder pattern":
 
-        >>> from graphtik import operation
+        >>> from graphtik import operation, varargs
 
-        >>> opb = operation(name='add_op')
-        >>> opb.withset(needs=['a', 'b'])
-        operation(name='add_op', needs=['a', 'b'], provides=[], fn=None)
-        >>> opb.withset(provides='SUM', fn=sum)
-        operation(name='add_op', needs=['a', 'b'], provides=['SUM'], fn='sum')
+        >>> op = operation()
+        >>> op
+        <function FunctionalOperation.withset at ...
 
-    You may keep calling ``withset()`` till you invoke a final ``__call__()``
-    on the builder;  then you get the actual :class:`.FunctionalOperation` instance:
+    That's a "fancy decorator".
 
-        >>> # Create `Operation` and overwrite function at the last moment.
-        >>> opb(sum)
-        FunctionalOperation(name='add_op', needs=['a', 'b'], provides=['SUM'], fn='sum')
+        >>> op = op.withset(needs=['a', 'b'])
+        >>> op
+        FunctionalOperation(name=None, needs=['a', 'b'], provides=[], fn=None)
 
-    .. Tip::
-        Remember to call once more the builder class at the end, to get the actual
-        operation instance.
+    If you call an operation with `fn` un-initialized, it will scream:
 
+        >>> op.compute({"a":1, "b": 2})
+        Traceback (most recent call last):
+        ValueError: Operation was not yet provided with a callable `fn`!
+
+    You may keep calling ``withset()`` until a valid operation instance is returned,
+    and compute it:
+
+        >>> op = op.withset(needs=['a', 'b'],
+        ...                 provides='SUM', fn=lambda a, b: a + b)
+        >>> op
+        FunctionalOperation(name='<lambda>', needs=['a', 'b'], provides=['SUM'], fn='<lambda>')
+        >>> op.compute({"a":1, "b": 2})
+        {'SUM': 3}
+
+        >>> op.withset(fn=lambda a, b: a * b).compute({'a': 2, 'b': 5})
+        {'SUM': 10}
     """
+    kw = {k: v for k, v in locals().items() if v is not None and k != "self"}
+    op = FunctionalOperation(**kw)
 
-    def __init__(
-        self,
-        fn: Callable = None,
-        *,
-        name=None,
-        needs: Items = None,
-        provides: Items = None,
-        aliases: Mapping = None,
-        rescheduled=None,
-        endured=None,
-        parallel=None,
-        marshalled=None,
-        returns_dict=None,
-        node_props: Mapping = None,
-    ):
-        kw = _dict_without(locals(), "self")
-        vars(self).update(kw)
-        # To check `fn` callable.
-        self.withset(**kw)
+    if "fn" in kw:
+        # Either used as a "naked" decorator (without any arguments)
+        # or not used as decorator at all (manually called and passed in `fn`) .
+        return op
 
-    def withset(
-        self,
-        *,
-        fn: Callable = None,
-        name=None,
-        needs: Items = None,
-        provides: Items = None,
-        aliases: Mapping = None,
-        rescheduled=None,
-        endured=None,
-        parallel=None,
-        marshalled=None,
-        returns_dict=None,
-        node_props: Mapping = None,
-    ) -> "operation":
-        """See :class:`operation` for arguments here."""
-        if fn is not None:
-            if not callable(fn):
-                raise ValueError(
-                    f"`fn` arg must be callable, was {type(fn).__name__}!"
-                    f"\n  did you mean? operation(name={fn}, ..."
-                )
-            self.fn = fn
-        if name is not None:
-            self.name = name
-        if needs is not None:
-            self.needs = needs
-        if provides is not None:
-            self.provides = provides
-        if aliases is not None:
-            self.aliases = aliases
-        if rescheduled is not None:
-            self.rescheduled = rescheduled
-        if endured is not None:
-            self.endured = endured
-        if parallel is not None:
-            self.parallel = parallel
-        if marshalled is not None:
-            self.marshalled = marshalled
-        if returns_dict is not None:
-            self.returns_dict = returns_dict
-        if node_props is not None:
-            self.node_props = node_props
+    @wraps(op.withset)
+    def decorator(*args, **kw):
+        return op.withset(*args, **kw)
 
-        return self
+    # Allow the decorator to support the builder-pattern.
+    decorator.withset = op.withset
 
-    def __call__(
-        self,
-        fn: Callable = None,
-        *,
-        name=None,
-        needs: Items = None,
-        provides: Items = None,
-        aliases: Mapping = None,
-        rescheduled=None,
-        endured=None,
-        parallel=None,
-        marshalled=None,
-        returns_dict=None,
-        node_props: Mapping = None,
-    ) -> FunctionalOperation:
-        """
-        This enables ``operation`` to act as a decorator or as a functional
-        operation, for example::
-
-            @operator(name='myadd1', needs=['a', 'b'], provides=['c'])
-            def myadd(a, b):
-                return a + b
-
-        or::
-
-            def myadd(a, b):
-                return a + b
-            operator(name='myadd1', needs=['a', 'b'], provides=['c'])(myadd)
-
-        :param fn:
-            The function to be used by this ``operation``.
-
-        :return:
-            Returns an operation class that can be called as a function or
-            composed into a computation graph.
-        """
-        self.withset(**_dict_without(locals(), "self"))
-
-        return FunctionalOperation(**vars(self))
-
-    def __repr__(self):
-        """
-        Display more informative names for the Operation class
-        """
-        needs = aslist(self.needs, "needs")
-        provides = aslist(self.provides, "provides")
-        aliases = aslist(self.aliases, "aliases")
-        aliases = f", aliases={aliases!r}" if aliases else ""
-        fn_name = self.fn and func_name(self.fn, None, mod=0, fqdn=0, human=0)
-        nprops = f", x{len(self.node_props)}props" if self.node_props else ""
-        resched = "?" if self.rescheduled else ""
-        endured = "!" if self.endured else ""
-        parallel = "|" if self.parallel else ""
-        marshalled = "$" if self.marshalled else ""
-        return (
-            f"operation{endured}{parallel}{marshalled}(name={self.name!r}, needs={needs!r}, "
-            f"provides={provides!r}{resched}{aliases}, fn={fn_name!r}{nprops})"
-        )
+    return decorator
