@@ -280,7 +280,7 @@ class _GraphtikTestDirective(extdoctest.TestDirective):
         img_attrs = {k: v for k, v in options.items() if k in _img_options}
         # TODO: emulate sphinx-processing for image width & height attrs.
         image = dynaimage(**img_attrs)
-        image.source, dynaimage.line = location
+        image.source, image.line = location
         image["classes"].extend(options.get("class", "").split())
         #  TODO: TCs for zooamble-SVGs options & configs.
         if "svg" in img_format:
@@ -493,7 +493,7 @@ def setup(app: Sphinx):
     app.add_config_value("graphtik_plot_keywords", {}, "html", [cabc.Mapping])
     app.add_config_value("graphtik_zoomable", True, "html", [bool])
     app.add_config_value("graphtik_warning_is_error", False, "html", [bool])
-    app.add_config_value("graphtik_save_dot_files", False, "html", [bool])
+    app.add_config_value("graphtik_save_dot_files", None, "html", [bool])
     app.add_config_value(
         "graphtik_zoomable_options",
         "{controlIconsEnabled: true, fit: true}",
