@@ -148,14 +148,14 @@ class _ScreamingOperation(op.Operation):
     [
         # NO old-stuff Operation(fn=_jetsamed_fn, name="test", needs="['a']", provides=[]),
         (
-            fnt.partial(
+            lambda: fnt.partial(
                 operation(name="test", needs=["a"], provides=["b"])(_scream).compute,
                 named_inputs={"a": 1},
             ),
             "outputs provides aliases results_fn results_op operation args".split(),
         ),
         (
-            fnt.partial(
+            lambda: fnt.partial(
                 network.ExecutionPlan(*([None] * 6))._handle_task,
                 op=_ScreamingOperation(),
                 solution=Solution(MagicMock(), {}),
@@ -168,6 +168,7 @@ class _ScreamingOperation(op.Operation):
 )
 def test_jetsam_sites_screaming_func(acallable, expected_jetsam):
     # Check jetsams when the underlying function fails.
+    acallable = acallable()
     with pytest.raises(Exception, match="ABC") as excinfo:
         acallable()
 
@@ -181,27 +182,29 @@ def test_jetsam_sites_screaming_func(acallable, expected_jetsam):
     [
         # NO old-stuff Operation(fn=_jetsamed_fn, name="test", needs="['a']", provides=[]),
         (
-            fnt.partial(operation(_scream, name="test")().compute, named_inputs=None),
+            lambda: fnt.partial(
+                operation(_scream, name="test").compute, named_inputs=None
+            ),
             "outputs provides aliases results_fn results_op operation args".split(),
         ),
         (
-            fnt.partial(
+            lambda: fnt.partial(
                 network.ExecutionPlan(*([None] * 6))._handle_task,
-                op=operation(_scream, name="Ah!")(),
+                op=operation(_scream, name="Ah!"),
                 solution=Solution(MagicMock(), {}),
                 future=_OpTask(_ScreamingOperation(), {}, "solid"),
             ),
             "plan solution task".split(),
         ),
         (
-            fnt.partial(
+            lambda: fnt.partial(
                 network.ExecutionPlan(*([None] * 6)).execute, named_inputs=None
             ),
             ["solution"],
         ),
         (
-            fnt.partial(
-                NetworkOperation([operation(str)()], "name").compute,
+            lambda: fnt.partial(
+                NetworkOperation([operation(str)], "name").compute,
                 named_inputs=None,
                 outputs="bad",
             ),
@@ -211,6 +214,7 @@ def test_jetsam_sites_screaming_func(acallable, expected_jetsam):
 )
 def test_jetsam_sites_scream(acallable, expected_jetsam):
     # Check jetsams when the site fails.
+    acallable = acallable()
     with pytest.raises(Exception) as excinfo:
         acallable()
 
@@ -482,7 +486,7 @@ def test_func_sourcelines_func(fn):
     exp = "def _foo():\n    pass"
     got = base.func_sourcelines(fn, human=1)
     assert "".join(got[0]).strip() == exp
-    assert got[1] == 225
+    assert got[1] == 229
 
 
 @pytest.mark.parametrize(
@@ -499,7 +503,7 @@ def test_func_sourcelines_method(fn):
     exp = "def foo(self):\n        pass"
     got = base.func_sourcelines(fn, human=1)
     assert "".join(got[0]).strip() == exp
-    assert got[1] == 230
+    assert got[1] == 234
 
 
 @pytest.mark.parametrize("fn", [eval, fnt.partial(eval)])
