@@ -9,12 +9,14 @@ import sys
 import time
 import types
 from functools import partial
+from itertools import cycle
 from multiprocessing import Pool, cpu_count
 from multiprocessing import dummy as mp_dummy
 from multiprocessing import get_context
 from operator import add, floordiv, mul, sub
 from pprint import pprint
 from unittest.mock import MagicMock
+from typing import Tuple
 
 import pytest
 
@@ -32,11 +34,9 @@ from graphtik import (
     operation,
     operations_endured,
     operations_reschedullled,
-    optional,
-    sideffect,
     tasks_marshalled,
-    vararg,
 )
+from graphtik.modifiers import optional, sideffect, sol_sideffect, vararg
 from graphtik.netop import NetworkOperation
 from graphtik.network import Solution
 from graphtik.op import Operation
@@ -167,7 +167,9 @@ def test_smoke_test():
     # Sum operation, early-bind compute function
     sum_op_factory = operation(add)
 
-    sum_op3 = sum_op_factory(name="sum_op3", needs=["a", "b"], provides="sum_ab2")
+    sum_op3 = sum_op_factory.withset(
+        name="sum_op3", needs=["a", "b"], provides="sum_ab2"
+    )
 
     # sum_op3 is callable
     assert sum_op3(5, 6) == 11
@@ -394,10 +396,14 @@ def test_network_merge_in_doctests():
 
 
 def test_aliases(exemethod):
+    aliased = operation(lambda: "A", name="op1", provides="a", aliases={"a": "b"})
+    assert aliased.provides == ("a",)
+    assert aliased.op_provides == ("a", "b")
+
     op = compose(
         "test_net",
-        operation(lambda: "A", name="op1", provides="a", aliases={"a": "b"})(),
-        operation(lambda x: x * 2, name="op2", needs="b", provides="c")(),
+        aliased,
+        operation(lambda x: x * 2, name="op2", needs="b", provides="c"),
         parallel=exemethod,
     )
     assert op() == {"a": "A", "b": "A", "c": "AA"}
@@ -557,8 +563,6 @@ def test_pruning_not_overrides_given_intermediate(exemethod):
 
 
 def test_pruning_multiouts_not_override_intermediates1(exemethod):
-    # Test #25: v.1.2.4 overwrites intermediate data when a previous operation
-    # must run for its other outputs (outputs asked or not)
     pipeline = compose(
         "graph",
         operation(name="must run", needs=["a"], provides=["overridden", "calced"])(
@@ -568,54 +572,33 @@ def test_pruning_multiouts_not_override_intermediates1(exemethod):
         parallel=exemethod,
     )
 
+    ## Overwritten values used as downstream inputs.
+    #
     inp1 = {"a": 5, "overridden": 1}
     inp2 = {"a": 5, "overridden": 1, "c": 2}
-    exp = {"a": 5, "overridden": 5, "calced": 10, "asked": 11}
+    exp = {"a": 5, "overridden": 5, "calced": 10, "asked": 15}
     exp2 = filtdict(exp, "asked")
 
-    # FAILs
-    # - on v1.2.4 with (overridden, asked) = (5, 15) instead of (1, 11)
-    # - on #18(unsatisfied) + #23(ordered-sets) like v1.2.4.
-    # FIXED on #26
-    # - on v4.0.0 (overridden, asked) := (5, 11)
     solution = pipeline.compute(inp1)
     assert solution == exp
     assert solution.overwrites == {"overridden": [5, 1]}
-
-    # FAILs
-    # - on v1.2.4 with KeyError: 'e',
-    # - on #18(unsatisfied) + #23(ordered-sets) with empty result.
-    # FIXED on #26
-    solution = pipeline.compute(inp2, "asked")
-    assert solution == exp2
-    assert solution.overwrites == {}
-
-    ## Test OVERWRITES
-    #
-    solution = pipeline.compute(inp1)
-    assert solution == exp
-    assert solution.overwrites == {"overridden": [5, 1]}
-
     # Check plotting Overwrites.
     assert "SkyBlue" in str(solution.plot())
 
-    solution = pipeline.compute(inp1, "asked")
+    solution = pipeline.compute(inp2, "asked")
+    assert solution == exp2
     assert solution.overwrites == {}
+    # Check not plotting Overwrites.
+    assert "SkyBlue" not in str(solution.plot())
 
+    solution = pipeline.compute(inp1, "asked")
+    assert solution == exp2
+    assert solution.overwrites == {}
     # Check not plotting Overwrites.
     assert "SkyBlue" not in str(solution.plot())
 
 
-@pytest.mark.xfail(
-    sys.version_info < (3, 6),
-    reason="PY3.5- have unstable dicts."
-    "E.g. https://travis-ci.org/ankostis/graphtik/jobs/595841023",
-)
 def test_pruning_multiouts_not_override_intermediates2(exemethod):
-    # Test #25: v.1.2.4 overrides intermediate data when a previous operation
-    # must run for its other outputs (outputs asked or not)
-    # SPURIOUS FAILS in < PY3.6 due to unordered dicts,
-    # eg https://travis-ci.org/ankostis/graphtik/jobs/594813119
     pipeline = compose(
         "pipeline",
         operation(name="must run", needs=["a"], provides=["overridden", "e"])(
@@ -627,18 +610,10 @@ def test_pruning_multiouts_not_override_intermediates2(exemethod):
     )
 
     inputs = {"a": 5, "overridden": 1, "c": 2}
-    exp = {"a": 5, "overridden": 5, "c": 2, "d": 3, "e": 10, "asked": 30}
-    # FAILs
-    # - on v1.2.4 with (overridden, asked) = (5, 70) instead of (1, 30)
-    # - on #18(unsatisfied) + #23(ordered-sets) like v1.2.4.
-    # FIXED on #26
-    # - on v4.0.0 (overridden, asked) := (5, 30)
+    exp = {"a": 5, "overridden": 5, "c": 2, "e": 10, "d": 7, "asked": 70}
+
     assert pipeline(**inputs) == exp
-    # FAILs
-    # - on v1.2.4 with KeyError: 'e',
-    # - on #18(unsatisfied) + #23(ordered-sets) with empty result.
     assert pipeline.compute(inputs, "asked") == filtdict(exp, "asked")
-    # FIXED on #26
 
     ## Test OVERWRITES
     #
@@ -698,13 +673,13 @@ def test_same_outputs_operations_order():
     subadd = compose("sub_add", op2, op1)
 
     inp = {"a": 3, "b": 1}
-    # assert addsub(**inp) == {"a": 3, "b": 1, "ab": 4}
-    # assert addsub.compute(inp, "ab") == {"ab": 4}
-    # assert subadd(**inp) == {"a": 3, "b": 1, "ab": 2}
-    # assert subadd.compute(inp, "ab") == {"ab": 2}
+    assert addsub(**inp) == {"a": 3, "b": 1, "ab": 2}
+    assert addsub.compute(inp, "ab") == {"ab": 2}
+    assert subadd(**inp) == {"a": 3, "b": 1, "ab": 4}
+    assert subadd.compute(inp, "ab") == {"ab": 4}
 
     # ## Check it does not duplicate evictions
-    # assert len(subadd.last_plan.steps) == 4
+    assert len(subadd.last_plan.steps) == 4
 
     ## Add another step to test evictions
     #
@@ -714,20 +689,20 @@ def test_same_outputs_operations_order():
 
     # Notice that `ab` assumed as 2 for `AB` but results in `2`
     solution = addsub.compute(inp)
-    assert solution == {"a": 3, "b": 1, "ab": 2, "AB": 4}
+    assert solution == {"a": 3, "b": 1, "ab": 2, "AB": 2}
     assert solution.overwrites == {"ab": [2, 4]}
     solution = addsub.compute(inp, "AB")
-    assert solution == {"AB": 4}
-    assert solution.overwrites == {}
-
-    solution = subadd.compute(inp)
-    assert solution == {"a": 3, "b": 1, "ab": 4, "AB": 2}
-    assert solution.overwrites == {"ab": [4, 2]}
-    solution = subadd.compute(inp, "AB")
     assert solution == {"AB": 2}
     assert solution.overwrites == {}
 
-    assert subadd.compute(inp, "AB") == {"AB": 2}
+    solution = subadd.compute(inp)
+    assert solution == {"a": 3, "b": 1, "ab": 4, "AB": 4}
+    assert solution.overwrites == {"ab": [4, 2]}
+    solution = subadd.compute(inp, "AB")
+    assert solution == {"AB": 4}
+    assert solution.overwrites == {}
+
+    assert subadd.compute(inp, "AB") == {"AB": 4}
     assert len(subadd.last_plan.steps) == 6
 
 
@@ -1007,7 +982,7 @@ def test_sideffect_steps(exemethod, netop_sideffect1: NetworkOperation):
 
 def test_sideffect_NO_RESULT(caplog):
     sfx = sideffect("b")
-    op = operation(lambda: NO_RESULT, provides=sfx)()
+    op = operation(lambda: NO_RESULT, provides=sfx)
     netop = compose("t", op)
     sol = netop.compute({}, outputs=sfx)
     assert sol == {}
@@ -1016,24 +991,102 @@ def test_sideffect_NO_RESULT(caplog):
     ## If NO_RESULT were not translated,
     #  a warning of unknown out might have emerged.
     caplog.clear()
-    netop = compose("t", operation(lambda: 1, provides=sfx)())
+    netop = compose("t", operation(lambda: 1, provides=sfx))
     netop.compute({}, outputs=sfx)
     for record in caplog.records:
         if record.levelname == "WARNING":
             assert "Ignoring result(1) because no `provides`" in record.message
 
     caplog.clear()
-    netop = compose("t", operation(lambda: NO_RESULT, provides=sfx)())
+    netop = compose("t", operation(lambda: NO_RESULT, provides=sfx))
     netop.compute({}, outputs=sfx)
     for record in caplog.records:
         assert record.levelname != "WARNING"
 
 
-@pytest.mark.xfail(
-    sys.version_info < (3, 6),
-    reason="PY3.5- have unstable dicts."
-    "E.g. https://travis-ci.org/ankostis/graphtik/jobs/595793872",
-)
+@pytest.fixture
+def calc_prices_pipeline():
+    @operation(needs="order_items", provides=sol_sideffect("ORDER", "Items", "Prices"))
+    def new_order(items: list) -> "pd.DataFrame":
+        order = {"items": items}
+        # Pretend we get the prices from sales.
+        order["prices"] = list(range(1, len(order["items"]) + 1))
+        return order
+
+    @operation(
+        needs=[sol_sideffect("ORDER", "Items"), "vat rate"],
+        provides=sol_sideffect("ORDER", "VAT rates"),
+    )
+    def fill_in_vat_ratios(order: "pd.DataFrame", base_vat: float) -> "pd.DataFrame":
+        order["VAT_rates"] = [
+            v for _, v in zip(order["prices"], cycle((base_vat, 2 * base_vat)))
+        ]
+        return order
+
+    @operation(
+        needs=[sol_sideffect("ORDER", "Prices"), sol_sideffect("ORDER", "VAT rates")],
+        provides=[sol_sideffect("ORDER", "VAT", "Totals"), "vat owed"],
+    )
+    def finalize_prices(order: "pd.DataFrame") -> Tuple["pd.DataFrame", float]:
+        if "VAT_rates" in order:
+            order["VAT"] = [p * v for p, v in zip(order["prices"], order["VAT_rates"])]
+            order["totals"] = [p + v for p, v in zip(order["prices"], order["VAT"])]
+            vat_to_pay = sum(order["VAT"])
+        else:
+            order["totals"] = order["prices"][::]
+        return order, vat_to_pay
+
+    proc_order = compose(
+        "process order", new_order, fill_in_vat_ratios, finalize_prices
+    )
+    return proc_order
+
+
+def test_solution_sideffects_ok(calc_prices_pipeline):
+    sol = calc_prices_pipeline.compute(
+        {"order_items": "milk babylino toilet-paper".split(), "vat rate": 0.18}
+    )
+    print(sol)
+    assert sol == {
+        "order_items": ["milk", "babylino", "toilet-paper"],
+        "vat rate": 0.18,
+        "ORDER": {
+            "items": ["milk", "babylino", "toilet-paper"],
+            "prices": [1, 2, 3],
+            "VAT_rates": [0.18, 0.36, 0.18],
+            "VAT": [0.18, 0.72, 0.54],
+            "totals": [1.18, 2.7199999999999998, 3.54],
+        },
+        "vat owed": 1.44,
+    }
+
+
+def test_solution_sideffects_endured(calc_prices_pipeline):
+    ## Break `fill_in_vat_ratios()`.
+    #
+    @operation(
+        needs=[sol_sideffect("ORDER", "Items"), "vat rate"],
+        provides=sol_sideffect("ORDER", "VAT rates"),
+        endured=True,
+    )
+    def fill_in_vat_ratios(order: "pd.DataFrame", base_vat: float) -> "pd.DataFrame":
+        raise ValueError("EC transactions have no VAT!")
+
+    calc_prices_pipeline = compose(
+        calc_prices_pipeline.name, fill_in_vat_ratios, calc_prices_pipeline, merge=True
+    )
+
+    sol = calc_prices_pipeline.compute(
+        {"order_items": "milk babylino toilet-paper".split(), "vat rate": 0.18}
+    )
+    print(sol)
+    assert sol == {
+        "order_items": ["milk", "babylino", "toilet-paper"],
+        "vat rate": 0.18,
+        "ORDER": {"items": ["milk", "babylino", "toilet-paper"], "prices": [1, 2, 3]},
+    }
+
+
 def test_optional_per_function_with_same_output(exemethod):
     # Test that the same need can be both optional and not on different operations.
     #
@@ -1160,8 +1213,8 @@ def test_execution_endurance(exemethod, endurance, endured):
         opb = operation(
             scream, needs=["a", "b"], provides=["a+b", "c"], endured=endured
         )
-        scream1 = opb(name="scream1")
-        scream2 = opb(name="scream2")
+        scream1 = opb.withset(name="scream1")
+        scream2 = opb.withset(name="scream2")
         add1 = operation(name="add1", needs=["a", "b"], provides=["a+b"])(add)
         add2 = operation(name="add2", needs=["a+b", "b"], provides=["a+2b"])(add)
         canceled = operation(name="canceled", needs=["c"], provides="cc")(identity)
@@ -1191,32 +1244,32 @@ def test_execution_endurance(exemethod, endurance, endured):
         #
         dot = str(sol.plot())
         assert "LightCoral" in dot  # failed
-        assert "Grey" in dot  # Canceled
+        assert "#a9a9a9" in dot  # Canceled
 
 
 @pytest.mark.parametrize(
     "resched, rescheduled", [(None, True), (True, None), (None, True), (1, 1)]
 )
 def test_rescheduling(exemethod, resched, rescheduled):
-    canc = operation(lambda: None, name="canc", needs=["b"], provides="cc")()
+    canc = operation(lambda: None, name="canc", needs=["b"], provides="cc")
     op = compose(
         "netop",
-        operation(lambda: [1], name="op1", provides=["a", "b"], rescheduled=1)(),
+        operation(lambda: [1], name="op1", provides=["a", "b"], rescheduled=1),
         canc,
-        operation(lambda: NO_RESULT, name="op2", provides=["c"], rescheduled=1)(),
+        operation(lambda: NO_RESULT, name="op2", provides=["c"], rescheduled=1),
         operation(
             lambda *args: sum(args),
             name="op3",
             needs=["a", optional("b"), optional("c")],
             provides=["d"],
-        )(),
+        ),
         parallel=exemethod,
     )
     sol = op()
     assert sol == {"a": 1, "d": 1}
     assert list(sol.canceled) == [canc]
     dot = str(sol.plot())
-    assert "Grey" in dot  # Canceled
+    assert "#a9a9a9" in dot  # Canceled
     assert 'BORDER="4"' in dot  # Rescheduled
     assert "x2 partial-ops" in str(sol.check_if_incomplete())
     with pytest.raises(IncompleteExecutionError, match="x2 partial-ops"):
@@ -1227,8 +1280,8 @@ def test_rescheduling(exemethod, resched, rescheduled):
 
 
 def test_rescheduling_NO_RESULT(exemethod):
-    partial = operation(lambda: NO_RESULT, name="op1", provides=["a"], rescheduled=1)()
-    canc = operation(lambda: None, name="canc", needs="a", provides="b")()
+    partial = operation(lambda: NO_RESULT, name="op1", provides=["a"], rescheduled=1)
+    canc = operation(lambda: None, name="canc", needs="a", provides="b")
     op = compose("netop", partial, canc, parallel=exemethod)
     sol = op()
     assert canc in sol.canceled
