@@ -16,7 +16,7 @@ from graphtik.modifiers import optional
 from graphtik.netop import NetworkOperation
 from graphtik.plot import (
     Plotter,
-    Style,
+    Theme,
     active_plotter_plugged,
     get_active_plotter,
 )
@@ -72,6 +72,8 @@ def _striplines(s):
 
 
 def test_op_label_template_full():
+    theme_op_badges = plot.Theme.op_badge_styles
+    avail_badges = theme_op_badges["badge_styles"]
     kw = dict(
         op_name="the op",
         fn_name="the fn",
@@ -85,36 +87,51 @@ def test_op_label_template_full():
         fn_url='http://fn_url.com/quoto"and',
         fn_tooltip="<fn\ntooltip>",
         fn_link_target="_top",
+        badges=list(avail_badges),  # cspell: disable-line
+        **theme_op_badges,
     )
-    got = plot._render_template(plot.Style.op_template, **kw)
+    got = plot._render_template(plot.Theme.op_template, **kw)
     print(got)
     exp = """
         <<TABLE CELLBORDER="0" CELLSPACING="0" STYLE="rounded" BORDER="44" COLOR="red" BGCOLOR="wheat">
             <TR>
                 <TD BORDER="1" SIDES="b" ALIGN="left" TOOLTIP="&lt;op &quot; &#9; tooltip&gt;" HREF="http://op_url.com_label_" TARGET="_self"
                 ><FONT COLOR="blue"><B>OP:</B> <I>the op</I></FONT></TD>
+                <TD BORDER="1" SIDES="b"><TABLE BORDER="0" CELLBORDER="0" CELLSPACING="1" CELLPADDING="2">
+                        <TR><TD STYLE="rounded" HEIGHT="22" VALIGN="BOTTOM" BGCOLOR="#04277d" TITLE="endured(!)" TARGET="_self"
+                            ><FONT FACE="monospace" COLOR="white"><B>E</B></FONT></TD><TD STYLE="rounded" HEIGHT="22" VALIGN="BOTTOM" BGCOLOR="#fc89ac" TITLE="rescheduled(?)" TARGET="_self"
+                            ><FONT FACE="monospace" COLOR="white"><B>R</B></FONT></TD><TD STYLE="rounded" HEIGHT="22" VALIGN="BOTTOM" BGCOLOR="#b1ce9a" TITLE="parallel(|)" TARGET="_self"
+                            ><FONT FACE="monospace" COLOR="white"><B>P</B></FONT></TD><TD STYLE="rounded" HEIGHT="22" VALIGN="BOTTOM" BGCOLOR="#4e3165" TITLE="marshalled($)" TARGET="_self"
+                            ><FONT FACE="monospace" COLOR="white"><B>M</B></FONT></TD><TD STYLE="rounded" HEIGHT="22" VALIGN="BOTTOM" BGCOLOR="#cc5500" TITLE="returns_dict({})" TARGET="_self"
+                            ><FONT FACE="monospace" COLOR="white"><B>D</B></FONT></TD></TR>
+                    </TABLE></TD>
             </TR><TR>
-                <TD ALIGN="left" TOOLTIP="&lt;fn&#10;tooltip&gt;" HREF="http://fn_url.com/quoto_and" TARGET="_top"
+                <TD COLSPAN="2" ALIGN="left" TOOLTIP="&lt;fn&#10;tooltip&gt;" HREF="http://fn_url.com/quoto_and" TARGET="_top"
                 ><FONT COLOR="blue"><B>FN:</B> the fn</FONT></TD>
             </TR>
         </TABLE>>
         """
 
     assert _striplines(got) == _striplines(exp)
+
+    ## Check
     for k, v in [
-        (k, v) for k, v in kw.items() if "tooltip" not in k and "url" not in k
+        (k, v)
+        for k, v in kw.items()
+        if "tooltip" not in k and "url" not in k and "badge" not in k
     ]:
         assert v in got, (k, v)
 
 
 def test_op_label_template_empty():
-    got = plot._render_template(plot.Style.op_template)
+    got = plot._render_template(plot.Theme.op_template)
     print(got)
     exp = """
         <<TABLE CELLBORDER="0" CELLSPACING="0" STYLE="rounded">
             <TR>
                 <TD BORDER="1" SIDES="b" ALIGN="left" TARGET=""
                 ></TD>
+                <TD BORDER="1" SIDES="b"></TD>
             </TR>
         </TABLE>>
         """
@@ -122,15 +139,16 @@ def test_op_label_template_empty():
 
 
 def test_op_label_template_fn_empty():
-    got = plot._render_template(plot.Style.op_template, op_name="op", fn_name="fn")
+    got = plot._render_template(plot.Theme.op_template, op_name="op", fn_name="fn")
     print(got)
     exp = """
         <<TABLE CELLBORDER="0" CELLSPACING="0" STYLE="rounded">
             <TR>
                 <TD BORDER="1" SIDES="b" ALIGN="left" TARGET=""
                 ><B>OP:</B> <I>op</I></TD>
+                <TD BORDER="1" SIDES="b"></TD>
             </TR><TR>
-                <TD ALIGN="left" TARGET=""
+                <TD COLSPAN="2" ALIGN="left" TARGET=""
                 ><B>FN:</B> fn</TD>
             </TR>
         </TABLE>>
@@ -152,13 +170,14 @@ def test_op_label_template_nones():
         fn_tooltip=None,
         fn_link_target=None,
     )
-    got = plot._render_template(plot.Style.op_template, **kw)
+    got = plot._render_template(plot.Theme.op_template, **kw)
     print(got)
     exp = """
         <<TABLE CELLBORDER="0" CELLSPACING="0" STYLE="rounded">
             <TR>
                 <TD BORDER="1" SIDES="b" ALIGN="left" TARGET=""
                 ></TD>
+                <TD BORDER="1" SIDES="b"></TD>
             </TR>
         </TABLE>>
         """
@@ -212,6 +231,7 @@ def test_plot_formats(pipeline, tmp_path):
         raise AssertionError("Failed pydot formats: %s" % "".join(sorted(dupe_errs)))
 
 
+@active_plotter_plugged(Plotter(include_steps=True))
 def test_plotters_hierarchy(pipeline: NetworkOperation, inputs, outputs):
     # Plotting original network, no plan.
     base_dot = str(pipeline.plot(inputs=inputs, outputs=outputs))
@@ -325,73 +345,91 @@ def test_plot_legend(pipeline, tmp_path):
 
 def test_style_Ref():
     s = plot.Ref("arch_url")
-    assert s.target == "https://graphtik.readthedocs.io/en/latest/arch.html"
-    assert repr(s) == "Ref(<class 'graphtik.plot.Style'>, 'arch_url')"
+    assert s.resolve(Theme) == "https://graphtik.readthedocs.io/en/latest/arch.html"
+    assert repr(s) == "Ref('arch_url')"
 
     class C:
         arch_url = "1"
 
-    s = s.rebased(C)
+    s = s.resolve(C)
     assert str(s) == "1"
-    assert (
-        repr(s) == "Ref(<class 'test.test_plot.test_style_Ref.<locals>.C'>, 'arch_url')"
-    )
 
     r = plot.Ref("resched_thickness")  # int target
     str(r)  # should not scream
-    assert r.target == 4
-    assert repr(r) == "Ref(<class 'graphtik.plot.Style'>, 'resched_thickness')"
+    assert r.resolve(Theme) == 4
+
+
+def test_Theme_with_set():
+    p = Theme()
+    p2 = p.with_set()
+    assert p is not p2
+
+    p3 = p.with_set(include_steps=True)
+    assert p3.include_steps
+    assert not p.include_steps
+
+
+def test_Plotter_with_styles():
+    p = Plotter()
+    p2 = p.with_styles()
+    assert p is not p2
+    assert p.default_theme is not p2.default_theme
+
+    p3 = p.with_styles(include_steps=True)
+    assert p3.default_theme.include_steps
+    assert not p.default_theme.include_steps
 
 
 def test_plotter_customizations(pipeline, monkeypatch):
     ## default URL
     #
-    url = Style.kw_legend["URL"]
+    url = Theme.kw_legend["URL"]
     dot = str(pipeline.plot())
-    assert url in dot
+    assert url in dot, dot
 
     ## New active_plotter
     #
-    with active_plotter_plugged(Plotter(style=Style(kw_legend={"URL": None}))):
+    with active_plotter_plugged(Plotter(theme=Theme(kw_legend={"URL": None}))):
         dot = str(pipeline.plot())
-        assert url not in dot
+        assert "legend" not in dot, dot
+        assert url not in dot, dot
 
         ## URL --> plotter in args
         #
         url1 = "http://example.1.org"
-        dot = str(pipeline.plot(plotter=Plotter(style=Style(kw_legend={"URL": url1}))))
-        assert url1 in dot
-        assert url not in dot
+        dot = str(pipeline.plot(plotter=Plotter(theme=Theme(kw_legend={"URL": url1}))))
+        assert url1 in dot, dot
+        assert url not in dot, dot
     dot = str(pipeline.plot())
-    assert url in dot
+    assert url in dot, dot
 
     url2 = "http://example.2.org"
-    with active_plotter_plugged(Plotter(style=Style(kw_legend={"URL": url2}))):
+    with active_plotter_plugged(Plotter(theme=Theme(kw_legend={"URL": url2}))):
         dot = str(pipeline.plot())
-        assert url2 in dot
-        assert url not in dot
+        assert url2 in dot, dot
+        assert url not in dot, dot
     dot = str(pipeline.plot())
-    assert url in dot
-    assert url1 not in dot
+    assert url in dot, dot
+    assert url1 not in dot, dot
 
     ## URL --> plotter in args
     #
-    dot = str(pipeline.plot(plotter=Plotter(style=Style(kw_legend={"URL": None}))))
-    assert url not in dot
+    dot = str(pipeline.plot(plotter=Plotter(theme=Theme(kw_legend={"URL": None}))))
+    assert "legend" not in dot, dot
 
-    dot = str(pipeline.plot(plotter=Plotter(style=Style(kw_legend={"URL": url2}))))
-    assert url2 in dot
-    assert url not in dot
+    dot = str(pipeline.plot(plotter=Plotter(theme=Theme(kw_legend={"URL": url2}))))
+    assert url2 in dot, dot
+    assert url not in dot, dot
 
 
 def test_plotter_customizations_ignore_class(pipeline, monkeypatch):
     # Class patches ignored
-    url = Style.kw_legend["URL"]
+    url = Theme.kw_legend["URL"]
     url_ignore = "http://foo.com"
-    monkeypatch.setitem(Style.kw_legend, "URL", url_ignore)
+    monkeypatch.setitem(Theme.kw_legend, "URL", url_ignore)
     dot = str(pipeline.plot())
-    assert url in dot
-    assert url_ignore not in dot
+    assert url in dot, dot
+    assert url_ignore not in dot, dot
 
 
 @pytest.mark.xfail(reason="jinja2 template fails pickling")
@@ -433,24 +471,26 @@ def test_node_dot_str0(dot_str_pipeline):
         digraph graph_ {
         fontname=italic;
         label=<graph>;
-        <edge> [shape=invhouse];
-        <digraph&#58; strict> [shape=invhouse];
+        <edge> [margin="0.02,0.02", shape=invhouse];
+        <digraph&#58; strict> [margin="0.02,0.02", shape=invhouse];
         <node> [label=<<TABLE CELLBORDER="0" CELLSPACING="0" STYLE="rounded">
             <TR>
                 <TD BORDER="1" SIDES="b" ALIGN="left" TOOLTIP="FunctionalOperation(name=&#x27;node&#x27;, needs=[&#x27;edge&#x27;, &#x27;digraph: strict&#x27;], provides=[&#x27;&lt;graph&gt;&#x27;], fn=&#x27;add&#x27;)" TARGET=""
                 ><B>OP:</B> <I>node</I></TD>
+                <TD BORDER="1" SIDES="b"></TD>
             </TR><TR>
-                <TD ALIGN="left" TOOLTIP="Same as a + b." TARGET=""
+                <TD COLSPAN="2" ALIGN="left" TOOLTIP="Same as a + b." TARGET=""
                 ><B>FN:</B> &lt;built-in function add&gt;</TD>
             </TR>
         </TABLE>>, shape=plain, tooltip=<node>];
-        <&lt;graph&gt;> [shape=house];
+        <&lt;graph&gt;> [margin="0.02,0.02", shape=house];
         <cu&#58;sto&#58;m> [label=<<TABLE CELLBORDER="0" CELLSPACING="0" STYLE="rounded">
             <TR>
                 <TD BORDER="1" SIDES="b" ALIGN="left" TOOLTIP="FunctionalOperation(name=&#x27;cu:sto:m&#x27;, needs=[&#x27;edge&#x27;, &#x27;digraph: strict&#x27;], provides=[&#x27;&lt;graph&gt;&#x27;], fn=&#x27;func&#x27;)" TARGET=""
                 ><B>OP:</B> <I>cu:sto:m</I></TD>
+                <TD BORDER="1" SIDES="b"></TD>
             </TR><TR>
-                <TD ALIGN="left" TOOLTIP="def func(a, b):&#10;    pass" TARGET=""
+                <TD COLSPAN="2" ALIGN="left" TOOLTIP="def func(a, b):&#10;    pass" TARGET=""
                 ><B>FN:</B> test.test_plot.func</TD>
             </TR>
         </TABLE>>, shape=plain, tooltip=<cu:sto:m>];
@@ -471,11 +511,6 @@ def test_node_dot_str0(dot_str_pipeline):
     sys.version_info < (3, 7), reason="PY3.6 - has different docstrings for builtins."
 )
 def test_node_dot_str1(dot_str_pipeline, monkeypatch):
-    style = get_active_plotter().style
-    monkeypatch.setattr(style, "py_item_url_format", "abc#%s")
-    monkeypatch.setattr(style, "op_link_target", "_self")
-    monkeypatch.setattr(style, "fn_link_target", "bad")
-
     ## Test node-hidding & Graph-overlaying.
     #
     overlay = nx.DiGraph()
@@ -483,25 +518,22 @@ def test_node_dot_str1(dot_str_pipeline, monkeypatch):
     overlay.add_node(hidden_op, _no_plot=True)
     overlay.graph["splines"] = "ortho"
 
-    sol = dot_str_pipeline.compute({"edge": 1, "digraph: strict": 2})
-    dot_str = str(sol.plot(graph=overlay))
-
-    print(dot_str)
     exp = """
         digraph solution_x5_nodes {
         fontname=italic;
         splines=ortho;
         subgraph "cluster_after pruning" {
         label="after pruning";
-        <edge> [fillcolor=wheat, shape=invhouse, style=filled, tooltip="(int) 1"];
-        <digraph&#58; strict> [fillcolor=wheat, shape=invhouse, style=filled, tooltip="(int) 2"];
-        <&lt;graph&gt;> [fillcolor=SkyBlue, shape=house, style=filled, tooltip=None];
+        <edge> [fillcolor=wheat, margin="0.02,0.02", shape=invhouse, style=filled, tooltip="(int) 1"];
+        <digraph&#58; strict> [fillcolor=wheat, margin="0.02,0.02", shape=invhouse, style=filled, tooltip="(int) 2"];
+        <&lt;graph&gt;> [fillcolor=SkyBlue, margin="0.02,0.02", shape=house, style=filled, tooltip="(None)"];
         <cu&#58;sto&#58;m> [label=<<TABLE CELLBORDER="0" CELLSPACING="0" STYLE="rounded" BGCOLOR="wheat">
             <TR>
                 <TD BORDER="1" SIDES="b" ALIGN="left" TOOLTIP="FunctionalOperation(name=&#x27;cu:sto:m&#x27;, needs=[&#x27;edge&#x27;, &#x27;digraph: strict&#x27;], provides=[&#x27;&lt;graph&gt;&#x27;], fn=&#x27;func&#x27;)" HREF="abc#{&#x27;dot_path&#x27;: &#x27;test.test_plot.func&#x27;, &#x27;posix_path&#x27;: &#x27;test/test_plot/func&#x27;}" TARGET="bad"
                 ><B>OP:</B> <I>cu:sto:m</I></TD>
+                <TD BORDER="1" SIDES="b"></TD>
             </TR><TR>
-                <TD ALIGN="left" TOOLTIP="def func(a, b):&#10;    pass" HREF="abc#{&#x27;dot_path&#x27;: &#x27;test.test_plot.func&#x27;, &#x27;posix_path&#x27;: &#x27;test/test_plot/func&#x27;}" TARGET="bad"
+                <TD COLSPAN="2" ALIGN="left" TOOLTIP="def func(a, b):&#10;    pass" HREF="abc#{&#x27;dot_path&#x27;: &#x27;test.test_plot.func&#x27;, &#x27;posix_path&#x27;: &#x27;test/test_plot/func&#x27;}" TARGET="bad"
                 ><B>FN:</B> test.test_plot.func</TD>
             </TR>
         </TABLE>>, shape=plain, tooltip=<cu:sto:m>];
@@ -513,5 +545,45 @@ def test_node_dot_str1(dot_str_pipeline, monkeypatch):
         legend [URL="https://graphtik.readthedocs.io/en/latest/_images/GraphtikLegend.svg", fillcolor=yellow, shape=component, style=filled, target=_top];
         }
         """
+
+    ## Theme-param.
+    #
+    sol = dot_str_pipeline.compute({"edge": 1, "digraph: strict": 2})
+    dot_str = str(
+        sol.plot(
+            graph=overlay,
+            theme=Theme(
+                py_item_url_format="abc#%s",
+                op_link_target="_self",
+                fn_link_target="bad",
+            ),
+        )
+    )
+
+    print(dot_str)
+    assert _striplines(dot_str) == _striplines(exp)
+    assert "<node>" not in dot_str
+
+    ## Plotter-param
+    #
+    dot_str = str(
+        sol.plot(
+            graph=overlay,
+            plotter=get_active_plotter().with_styles(
+                py_item_url_format="abc#%s",
+                op_link_target="_self",
+                fn_link_target="bad",
+            ),
+        )
+    )
+    assert _striplines(dot_str) == _striplines(exp)
+    assert "<node>" not in dot_str
+
+    ## (last one) Patch theme directly on default active-plotter.
+    #
+    theme = get_active_plotter().default_theme
+    monkeypatch.setattr(theme, "py_item_url_format", "abc#%s")
+    monkeypatch.setattr(theme, "op_link_target", "_self")
+    monkeypatch.setattr(theme, "fn_link_target", "bad")
     assert _striplines(dot_str) == _striplines(exp)
     assert "<node>" not in dot_str
